/*
 * Created on 01.11.2006
 */
package core;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import core.structures.Config;
import core.structures.Header;
import core.structures.Message;
import core.structures.Part;
import core.util.StringTools;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 *
 */
public class Pop3Connection {

	/**
	 * Establishes a Socket connection to the local runnig Server
	 */

	static Logger	logger	= Logger.getLogger(Pop3Connection.class);

	/**
	 * gets the body from a message
	 */
	private static String getBody(String[] msg) {
		for (int i = 0; i < msg.length; i++) {
			if (msg[i].length() < 1) {
				StringBuilder sb = new StringBuilder();
				for (int j = i + 1; j < msg.length; j++) {
					sb.append(msg[j]).append("\n");
				}
				return sb.toString();
			}
		}
		return "";
	}

	/**
	 * gets the header from a message
	 * 
	 * @param msg
	 */
	private static String getHeader(String[] msg) {
		for (int i = 0; i < msg.length; i++) {
			if (msg[i].length() < 1) {
				ArrayList<String> hd = new ArrayList<String>();
				for (int j = 0; j < i; j++) {
					hd.add(msg[j]);
				}
				// extracting the String out of the array
				return StringTools.arrayToString(Header.unfoldHeaderLines(hd));
			}
		}
		return "";
	}

	/**
	 * Reads a Mail from the temporary location
	 * 
	 * @param offset
	 *            the number of the mail to parse. is equal to the pop3-index id
	 * @return the parsed message
	 */
	public static Message getMailFile(int offset) {
		String[] rawMail = null;
		ArrayList<String> readBuffer = new ArrayList<String>();
		File cacheFile = new File(Config.CACHE_BASE_FILENAME + offset + Config.CACHE_EXTENSION);
		if (cacheFile.exists() && cacheFile.canRead()) {
			FileReader fr = null;
			BufferedReader br = null;
			try {
				fr = new FileReader(cacheFile);
				br = new BufferedReader(fr);
				String buffer = null;
				while ((buffer = br.readLine()) != null) {
					readBuffer.add(buffer);
				}
			} catch (FileNotFoundException e) {
				logger.warn("getMailFile - File not found! - Message #" + offset);
				return null;
			} catch (IOException e) {
				logger.warn("getMailFile - IO Error! - Message #" + offset);
				return null;
			} finally {
				try {
					fr.close();
					br.close();
				} catch (IOException e) {
					;
				}
			}
			if (readBuffer.size() < 1) {
				logger.trace("ArrayList is empty! - Message #" + offset);
				return null;
			}
			rawMail = new String[readBuffer.size()];
			rawMail = readBuffer.toArray(rawMail);
			if (rawMail == null) {
				logger.trace("rawMail is empty! - Message #" + offset);
				return null;
			}
			Message msg = new Message();
			Header hd = new Header();
			Part pt = null;
			hd.setRawHeader(getHeader(rawMail));
			if (hd.getContentType().toLowerCase().startsWith(Header.CONTENT_PREFIX_MULTIPART.toLowerCase())) {
				ArrayList<String[]> parts = getParts(rawMail, hd.getContentMultipartBoundary());
				// get Headers und Body for all parts
				for (int i = 0; i < parts.size(); i++) {
					pt = new Part();
					Header hdi = new Header();
					hdi.setRawHeader(getHeader(parts.get(i)));
					hdi.setDate(hd.getDate());
					if (hdi.getContentTypeCharset().length() < 1)
						hdi.setContentTypeCharset(hd.getContentTypeCharset());
					pt.setHeader(hdi);
					pt.setRawText(getBody(parts.get(i)));
					msg.addPart(pt);
				}
			} else { // get header a single part
				pt = new Part();
				Header hdi = new Header();
				hdi.setDate(hd.getDate());
				hdi.setContentType(hd.getContentType());
				hdi.setContentTransferEncoding(hd.getContentTransferEncoding());
				hdi.setContentTypeCharset(hd.getContentTypeCharset());
				pt.setRawText(getBody(rawMail));
				pt.setHeader(hdi);
				msg.addPart(pt);
			}
			msg.setHeader(hd);
			return msg;
		} else {
			logger.warn("File doesn't exist. Filename: " + cacheFile.getAbsolutePath());
			return null;
		}
	}

	/**
	 * Splits the given message at the boundary into the parts
	 * 
	 * @param msg
	 * @param boundary
	 * @return
	 */
	private static ArrayList<String[]> getParts(String[] msg, String boundary) {
		ArrayList<String[]> pts = new ArrayList<String[]>();
		ArrayList<String> pt = new ArrayList<String>();
		for (int i = 0; i < msg.length; i++) {
			// sometimes the boundary is preceeded by two -'s, so we'll accept both ways
			if (msg[i].startsWith(boundary + "--") || msg[i].startsWith("--" + boundary + "--")) {
				// this is the end of the multipart mail
				if (pt.size() > 0) {
					pts.add(pt.toArray(new String[pt.size()]));
				}
				pt = new ArrayList<String>();
				break;
			} else if (msg[i].startsWith(boundary) || msg[i].startsWith("--" + boundary)) {
				/*
				 * From RFC 2046: NOTE TO IMPLEMENTORS: Boundary string comparisons must compare the boundary value with
				 * the beginning of each candidate line. An exact match of the entire candidate line is not required; it
				 * is sufficient that the boundary appear in its entirety following the CRLF.
				 * 
				 * Thus we only compare the beginning of each line.
				 */
				pts.add(pt.toArray(new String[pt.size()]));
				pt = new ArrayList<String>();
			} else {
				pt.add(msg[i]);
			}
		}
		// add the last part to the list of parts
		if (pt.size() > 0) {
			pts.add(pt.toArray(new String[pt.size()]));
		}
		pts.remove(0); // remove the first part, it's just a blank line or
		// line-feed
		return pts;
	}

	private Config				config		= null;

	private OutputStream		os			= null;

	private OutputStreamWriter	osw			= null;

	private BufferedWriter		bw			= null;

	private InputStream			is			= null;

	private InputStreamReader	isr			= null;

	private BufferedReader		br			= null;

	private Socket				socket		= null;

	private String				recvBuffer	= "";

	private String				sendBuffer	= "";

	public Pop3Connection(Config config) {
		this.config = config;
	}

	/**
	 * Establishes the connection to the server
	 */
	public boolean connect() {
		try {
			this.socket = new Socket(config.getAddress(), config.getPort());
			this.os = this.socket.getOutputStream();
			this.is = this.socket.getInputStream();
			this.isr = new InputStreamReader(is);
			this.br = new BufferedReader(isr);
			this.osw = new OutputStreamWriter(os);
			this.bw = new BufferedWriter(osw);
			String recvBuffer = br.readLine();
			// if +OK
			if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
				return true; // server sent an success-message
			} else {
				logger.warn("Pop3Connection:connect() - Connection failed");
				return false; // server sent an error response or no response
			}
		} catch (IOException e) {
			logger.error("Pop3Connection:connect() - Connection couldn't established");
			return false; // connection couldn't established
		}
	}

	/**
	 * deletes a email at the server mailbox by a Delete-command
	 * 
	 * @return succes
	 */
	public boolean deleteMail(int n) {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_DELE + " " + n + Config.CRLF;
				logger.trace("Pop3Connection:deleteMail(" + n + ") - sendBuffer: " + sendBuffer);
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				logger.trace("Pop3Connection:deleteMail(" + n + ") - recvBuffer: " + recvBuffer);
				// +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					return true;
				} else {
					logger.trace("Pop3Connection:deleteMail(" + n + ") - dele command not acked");
					return false; // the server didn't like the dele command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return false; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return false; // buffer reader and writer are not setup, probably
			// connection wasn't called
		}
	}

	/**
	 * Disconnet to the server by sending a QUIT command
	 * 
	 */
	public void disconnect() {
		try {
			bw.write(Config.POP3_CMD_QUIT + Config.CRLF);
			bw.flush();
		} catch (IOException e) {
			; // don't care we're trying to disconnect anyway
		} finally {
			try {
				bw.close();
				br.close();
			} catch (IOException e1) {
				; // don't care we're in a catch block anyway
			}
		}

	}

	/**
	 * getting the list be sending a LIST command
	 */
	public String getList() {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_LIST + Config.CRLF;
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				// +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					String buffer = "";
					while (true) {
						recvBuffer = br.readLine();
						if (recvBuffer == null || recvBuffer.startsWith(".")) {
							break;
						} else {
							buffer += recvBuffer + "\n";
						}
					}
					return buffer;
				} else {
					logger.warn("Pop3Connection:getList() - list command not acked");
					return ""; // the server didn't like the stat command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return ""; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return ""; // buffer reader and writer are not setup, probably
			// connection wasn't called
		}
	}

	/**
	 * extracts the nth message out of the mailbox
	 */
	public Message getMailString(int n) {
		String[] rawMail = this.retrieveMailString(n);
		if (rawMail == null) {
			return null;
		}
		Message msg = new Message();
		Header hd = new Header();
		Part pt = null;
		hd.setRawHeader(getHeader(rawMail));
		// a multipart message multipart-message:
		if (hd.getContentType().toLowerCase().startsWith(Header.CONTENT_PREFIX_MULTIPART.toLowerCase())) {
			ArrayList<String[]> parts = getParts(rawMail, hd.getContentMultipartBoundary());
			// get Headers und Body for all parts
			for (int i = 0; i < parts.size(); i++) {
				pt = new Part();
				Header hdi = new Header();
				hdi.setRawHeader(getHeader(parts.get(i)));
				pt.setHeader(hdi);
				pt.setRawText(getBody(parts.get(i)));
				msg.addPart(pt);
			}
		} else { // get header a single part
			pt = new Part();
			Header hdi = new Header();
			hdi.setContentType(hd.getContentType());
			hdi.setContentTransferEncoding(hd.getContentTransferEncoding());
			pt.setRawText(getBody(rawMail));
			pt.setHeader(hdi);
			msg.addPart(pt);
		}
		msg.setHeader(hd);
		return msg;
	}

	/**
	 * @return number of mails on the server
	 */
	public int getNumberOfMails() {
		String buffer = "";
		buffer = this.getStat();
		StringTokenizer tokens = new StringTokenizer(buffer);
		if (tokens.hasMoreTokens()) {
			tokens.nextToken(); // +OK
		}
		if (tokens.hasMoreTokens()) {
			return Integer.parseInt(tokens.nextToken()); // The number of the
			// mails
		}
		return -1;
	}

	public int getSizeOfMails() {
		String buffer = "";
		buffer = this.getStat();
		StringTokenizer tokens = new StringTokenizer(buffer);
		if (tokens.hasMoreTokens()) {
			tokens.nextToken(); // +OK
		}
		if (tokens.hasMoreTokens()) {
			tokens.nextToken(); // The number of the mails
		}
		if (tokens.hasMoreTokens()) {
			return Integer.parseInt(tokens.nextToken()); // The size of the
			// mails
		}
		return -1;
	}

	/**
	 * client getting the state by sending a STAT command to the server
	 */
	private String getStat() {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_STAT + Config.CRLF;
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				// +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					return recvBuffer;
				} else {
					logger.warn("Pop3Connection:getStat() - Stat command not successfull");
					return ""; // the server didn't like the stat command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					logger.error("Pop3Connection:getStat() - Caught Exception, connection problem");
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return ""; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return ""; // buffer reader and writer are not setup, probably
			// connection wasn't called
		}
	}

	public boolean login() {
		return this.login(config.getUsername(), config.getPassword());
	}

	/**
	 * Server-Login authentification of the client
	 */
	public boolean login(String username, String password) {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_USER + " " + config.getUsername() + Config.CRLF;
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				// if +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					// gete Password
					sendBuffer = Config.POP3_CMD_PASS + " " + config.getPassword() + Config.CRLF;
					bw.write(sendBuffer);
					bw.flush();
					recvBuffer = br.readLine();
					if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
						return true; // the login to the server succeded
					} else {
						logger.warn("Pop3Connection:login() - Login failed. Password wrong?");
						return false; // the login faild. the server didn't
						// like our password
					}
				} else {
					logger.warn("Pop3Connection:login() - User command not acked");
					return false; // the server didn't like the user command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					logger.error("Pop3Connection:login() - Caught Exception, connection problem");
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return false; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return false; // buffer reader and writer are not setup, probably
			// connect wasn't called
		}
	}

	public boolean retrieveMailFile(int offset) {
		FileWriter fw = null;
		BufferedWriter bw = null;
		try {
			File tmpFile = new File(Config.CACHE_BASE_FILENAME + offset + Config.CACHE_EXTENSION);
			tmpFile.deleteOnExit(); // this is a temp file, delete it when the VM exits
			fw = new FileWriter(tmpFile);
			bw = new BufferedWriter(fw);
			String[] buffer = this.retrieveMailString(offset);
			if (buffer == null)
				return false;
			for (int i = 0; i < buffer.length; i++) {
				bw.write(buffer[i]);
				bw.newLine();
			}
			buffer = null;
		} catch (IOException e) {
			return false;
		} finally {
			try {
				bw.close();
				fw.close();
			} catch (IOException e) {
				;
			}
		}
		return true;
	}

	/**
	 * retreiving emails by sending a RETR command
	 * 
	 * @param n
	 * @return
	 */
	private String[] retrieveMailString(int n) {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_RETR + " " + n + Config.CRLF;
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					// logger.trace("Retrieving Mail (every dot is a line):");
					ArrayList<String> buffer = new ArrayList<String>();
					while (true) {
						recvBuffer = br.readLine();
						if (recvBuffer == null || recvBuffer.startsWith(".")) {
							break;
						} else {
							buffer.add(recvBuffer);
						}
					}
					// logger.trace("retrieveMailString("+ n +") - Returning ...");
					String[] returnBuffer = new String[buffer.size()];
					buffer.toArray(returnBuffer);
					return returnBuffer;
				} else {
					logger.warn("retrieveMailString(" + n + ") - retr command not acknowledeged - Error Msg: "
							+ recvBuffer);
					return null; // the server didn't like the retr command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return null; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return null; // buffer reader and writer are not setup, probably
			// connection wasn't called
		}
	}

	/**
	 * Tells the Server no operation is shall be processed
	 * 
	 */
	public void sendNoop() {
		if (br != null && bw != null) {
			try {
				sendBuffer = Config.POP3_CMD_NOOP + Config.CRLF;
				logger.trace("Pop3Connection:sendNoop() - sendBuffer: " + sendBuffer);
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				logger.trace("Pop3Connection:sendNoop() - recvBuffer: " + recvBuffer);
				// +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					logger.trace("NOOP-Success");
				} else {
					logger.trace("Pop3Connection:sendNoop() - dele command not acked");
				}
			} catch (IOException e) {
				try {
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
			}
		}
	}

	/**
	 * Tells the server to reset by sending a Reset command
	 */
	public boolean sendReset() {
		if (br != null && bw != null) {
			try {
				// command
				sendBuffer = Config.POP3_CMD_RSET + Config.CRLF;
				logger.trace("Pop3Connection:sendReset() - sendBuffer: " + sendBuffer);
				bw.write(sendBuffer);
				bw.flush();
				recvBuffer = br.readLine();
				logger.trace("Pop3Connection:sendReset() - recvBuffer: " + recvBuffer);
				// +OK
				if (recvBuffer.startsWith(Config.POP3_STATUS_SUCCESS)) {
					return true;
				} else {
					logger.trace("Pop3Connection:sendReset() - rset command not acked");
					return false; // the server didn't like the rset command,
					// something is wrong
				}
			} catch (IOException e) {
				try {
					bw.close();
					br.close();
				} catch (IOException e1) {
					; // don't care, becasue we're in a catch block already
					// and are just trying to cover our tracks
				}
				return false; // couldn't send or recieve data - probably the
				// connection was interrupted
			}
		} else {
			return false; // buffer reader and writer are not setup, probably
			// connection wasn't called
		}
	}

}
