package core.structures;

import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * this class contains a mail consisting of a header and several parts (in case of a multipart mail) or just one part in
 * case of an ordinary mail.
 * 
 * @author tex
 */
public class Message {

	static Logger			logger	= Logger.getLogger(Message.class);

	// A email contains a headers and different Parts of content

	private Header			header	= null;

	private ArrayList<Part>	parts	= null;

	public Message() {
		parts = new ArrayList<Part>();
	}

	/**
	 * 
	 * @param part
	 *            the part to set
	 * @return the success
	 */
	public synchronized boolean addPart(Part part) {
		return parts.add(part);
	}

	/**
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object msg) {
		// this objects can only equal if they are from the same type
		if (msg instanceof Message) {
			Message other = (Message) msg;
			if (other != null && this.header.equals(other.header)) {
				for (int i = 0; i < this.parts.size(); i++) {
					if (other.getPart(i) != null && !this.parts.get(i).equals(other.getPart(i))) {
						return false;
					}
				}
				return true;
			}
			return false;
		} else {
			return false;
		}
	}

	/**
	 * @return the header
	 */
	public Header getHeader() {
		return header;
	}

	public int getNumParts() {
		return parts.size();
	}

	public Part getPart(int n) {
		return parts.get(n);
	}

	public Part[] getParts() {
		return (Part[]) parts.toArray();
	}

	/**
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		int hashCode = 0;
		hashCode += this.getHeader().hashCode();
		for (int i = 0; i < this.getNumParts(); i++) {
			hashCode += this.getPart(i).hashCode();
		}
		return hashCode;
	}

	/**
	 * @param header
	 *            the header to set
	 */
	public void setHeader(Header header) {
		this.header = header;
	}

	/**
	 * @return a String that shows the Sender, receiver and subject of a email
	 */
	public String toString() {
		return "From: " + this.header.getFrom() + ", To: " + this.header.getTo() + ", Subject: "
				+ this.header.getSubject();
	}

}
