package core.structures;

import org.apache.log4j.Logger;

import core.util.Base64Decoder;
import core.util.Decode;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * This class contains a Part of a Multipart message
 * 
 * @author tex, flo
 * 
 */
public class Part {

	static Logger	logger	= Logger.getLogger(Part.class);

	private String	rawText	= "";

	private Header	header	= null;

	/**
	 * 
	 * @param prt
	 * @return
	 */
	public boolean equals(Part prt) {
		if (prt != null && this.header.equals(prt.getHeader()) && this.rawText.equals(prt.getRawText())) {
			return true;
		}
		return false;
	}

	/**
	 * Returns the decoded content of this part as a string, i.e. suitable for showing this to a user.
	 * 
	 * @return
	 */
	public String getContent() {
		if (this.header.getContentTransferEncoding().toLowerCase().equals(
				Header.CONTENT_TRANSER_ENCODING_QP.toLowerCase())) {
			return Decode.decodeQuotedPrintable(this.getRawText(), this.getHeader().getContentTypeCharset());
		} else if (this.header.getContentTransferEncoding().toLowerCase().equals(
				Header.CONTENT_TRANSER_ENCODING_BASE64.toLowerCase())) {
			return Decode.decodeBase64(this.getRawText(), this.getHeader().getContentTypeCharset());
		} else {
			return this.getRawText();
		}
	}

	public String getContentType() {
		return this.header.getContentType();
	}

	/**
	 * @return the header
	 */
	public Header getHeader() {
		return header;
	}

	/**
	 * Get the content of this message decoding it's (base64|qp|whatever)-encoded content as a byte-array, i.e. sutiable
	 * for writing binary data to a file.
	 * 
	 * Actually I think only base 64 will work this way ...
	 * 
	 * @return
	 */
	public byte[] getRawContent() {
		if (this.header.getContentTransferEncoding().toLowerCase().equals(
				Header.CONTENT_TRANSER_ENCODING_QP.toLowerCase())) {
			return Decode.decodeQuotedPrintable(this.getRawText(), "").getBytes();
		} else if (this.header.getContentTransferEncoding().toLowerCase().equals(
				Header.CONTENT_TRANSER_ENCODING_BASE64.toLowerCase())) {
			return Base64Decoder.decode(this.getRawText());
		} else {
			return this.getRawText().getBytes();
		}
	}

	/**
	 * @return the rawText
	 */
	public String getRawText() {
		return rawText;
	}

	/**
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		int hashCode = 0;
		hashCode += this.getHeader().hashCode();
		hashCode += this.getRawText().hashCode();
		return hashCode;
	}

	/**
	 * @param header
	 *            the header to set
	 */
	public void setHeader(Header header) {
		this.header = header;
	}

	/**
	 * @param rawText
	 *            the rawText to set
	 */
	public void setRawText(String rawText) {
		this.rawText = rawText;
	}

}
