package core.util;

import org.apache.log4j.Logger;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 * 
 */
public class Base64Decoder {

	static Logger			logger		= Logger.getLogger(Base64Decoder.class);

	/**
	 * The Base 64 charset as defined in the RFC
	 * Copy'n'Pasted from Rob Harders Public Domain Base64 Class.
	 */
	public static char[]	ALPHABET	= { (byte) 'A', (byte) 'B', (byte) 'C', (byte) 'D', (byte) 'E', (byte) 'F',
			(byte) 'G', (byte) 'H', (byte) 'I', (byte) 'J', (byte) 'K', (byte) 'L', (byte) 'M', (byte) 'N', (byte) 'O',
			(byte) 'P', (byte) 'Q', (byte) 'R', (byte) 'S', (byte) 'T', (byte) 'U', (byte) 'V', (byte) 'W', (byte) 'X',
			(byte) 'Y', (byte) 'Z', (byte) 'a', (byte) 'b', (byte) 'c', (byte) 'd', (byte) 'e', (byte) 'f', (byte) 'g',
			(byte) 'h', (byte) 'i', (byte) 'j', (byte) 'k', (byte) 'l', (byte) 'm', (byte) 'n', (byte) 'o', (byte) 'p',
			(byte) 'q', (byte) 'r', (byte) 's', (byte) 't', (byte) 'u', (byte) 'v', (byte) 'w', (byte) 'x', (byte) 'y',
			(byte) 'z', (byte) '0', (byte) '1', (byte) '2', (byte) '3', (byte) '4', (byte) '5', (byte) '6', (byte) '7',
			(byte) '8', (byte) '9', (byte) '+', (byte) '/' };

	public static byte[]	DECABETH	= new byte[128];

	static {
		// clear alphabet
		for (int i = 0; i < DECABETH.length; i++) {
			DECABETH[i] = -1;
		}
		// initialize with bytes
		for (int i = 0; i < 64; i++) {
			DECABETH[ALPHABET[i]] = (byte) i;
		}
	}

	/**
	 * replaces CRLF
	 */
	public static String prepareString(String msg) {
		msg = msg.replace("\n", "").replace("\r", "");
		return msg;
	}

	/**
	 * decodes a 64based-String into a byte-Array
	 */
	public static byte[] decode(String msg) {
		return decode(prepareString(msg).toCharArray());
	}

	/**
	 * Decodes a Base64 encoded charArray into a byteArray
	 */
	public static byte[] decode(char[] msg) {
		int inCnt = 0;
		int outCnt = 0;
		int msgLen = msg.length;
		int in[] = new int[4];
		int quad[] = new int[4];
		int triple[] = new int[3];
		// init vars
		for (int i = 0; i < in.length; i++) {
			in[i] = 0;
		}
		for (int i = 0; i < quad.length; i++) {
			quad[i] = 0;
		}
		for (int i = 0; i < triple.length; i++) {
			triple[i] = 0;
		}
		if (msgLen % 4 != 0) {
			logger
					.error("The length of the encoded message is not a multiple of 4. Decoding anyway, but result will be not correct!");
		}
		// calculate the length of the output buffer
		while (msgLen > 0 && msg[msgLen - 1] == '=') {
			msgLen--;
		}
		int outputLen = (msgLen * 3) / 4;
		byte[] outBuf = new byte[outputLen];
		// process the input "string"
		while (inCnt < msgLen) {
			in[0] = msg[inCnt++];
			in[1] = msg[inCnt++];
			if (inCnt < msgLen) {
				in[2] = msg[inCnt++];
			} else {
				in[2] = 'A';
			}
			if (inCnt < msgLen) {
				in[3] = msg[inCnt++];
			} else {
				in[3] = 'A';
			}
			for (int i = 0; i < in.length; i++) {
				if (in[i] > 127)
					logger.error("Illegal input character! Decoding anyway, but result will be not correct!");
			}
			for (int i = 0; i < quad.length; i++) {
				quad[i] = DECABETH[in[i]];
			}
			for (int i = 0; i < quad.length; i++) {
				if (quad[i] < 0)
					logger.error("Illegal input character! Decoding anyway, but result will be not correct!");
			}
			/* this is reduction from 4 chars to 3 bits */
			/* there are three possible ways of doing this, don't know which is best ... */

			/* this is our way */
			triple[0] = (quad[0] << 2) | (quad[1] >>> 4);
			triple[1] = ((quad[1] & 0xF) << 4) | (quad[2] >>> 2);
			triple[2] = ((quad[2] & 3) << 6) | quad[3];

			/* this one was suggested by a friend */
			// triple[1] = ((quad[1] << 4) & 0xF0) | quad[2] >> 2;
			// triple[1] = ((quad[1] << 4) & 0xF0) | quad[2] >> 2;
			// triple[2] = ((quad[2] << 6) & 0xC0) | quad[3];
			/* this one was presented by andreas hartl */
			// int t = quad[0] << 18 | quad[1] << 12 | quad[2] << 6 | quad[3] << 0;
			// triple[0] = t >> 16 & 0xFF;
			// triple[1] = t >> 8 & 0xFF;
			// triple[2] = t >> 0 & 0xFF;
			outBuf[outCnt++] = (byte) triple[0];
			if (outCnt < outputLen) {
				outBuf[outCnt++] = (byte) triple[1];
			}
			if (outCnt < outputLen) {
				outBuf[outCnt++] = (byte) triple[2];
			}
		}

		return outBuf;
	}
}
