package core.util;

import org.apache.log4j.Logger;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 *
 */
public class QuotedPrintable {

	static Logger				logger		= Logger.getLogger(QuotedPrintable.class);

	/*
	 * characterlist and hexList are constaining 128 ASCII and HEX chars. They are used to transform a char into the
	 * other represantation
	 */
	public static String[]		CHARLIST	= null;

	public static String[]		HEXLIST		= null;

	private final static int	CHARS		= 128;

	/**
	 * Generate the table for the mappings of the higher-valued chars to their replacements
	 * 
	 */
	static {
		CHARLIST = new String[CHARS];
		HEXLIST = new String[CHARS];
		// initialize the ASCII and HEX-Values in the String Arrays
		for (int i = 0; i < 127; i++) {
			CHARLIST[i] = String.valueOf((char) (i + 128));
			HEXLIST[i] = "=" + (Integer.toHexString(i + 128).toUpperCase());
		}
		CHARLIST[127] = "=";
		HEXLIST[127] = "=3D";
	}

	/**
	 * 
	 * @param msg
	 *            Transfroms a String from hexadecimal values into a ASCII-decoded String by replacing all chars of the
	 *            characterList and the hexList
	 */
	public static String decode(String msg, String charset) {
		String decoded = msg;
		for (int i = 0; i < QuotedPrintable.CHARLIST.length; i++) {
			decoded = decoded.replaceAll(QuotedPrintable.HEXLIST[i], QuotedPrintable.CHARLIST[i]);
		}
		decoded = decoded.replaceAll("=\n", "");
		return decoded;
	}

	/**
	 * 
	 * @param hex
	 * @return Char of the hex-value
	 */
	public static String getCharFromHex(String hex) {
		for (int i = 0; i < CHARLIST.length; i++) {
			if (HEXLIST[i] == hex)
				return CHARLIST[i];
		}
		return "";
	}

	/**
	 * Transforms a ASCII character into a Hex-char
	 */
	public static String getHexFromChar(String character) {
		for (int i = 0; i < HEXLIST.length; i++) {
			if (CHARLIST[i] == character)
				return HEXLIST[i];
		}
		return "";
	}
}
