package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.text.html.HTMLEditorKit;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import core.structures.Config;
import core.structures.Mailbox;
import core.structures.Message;
import core.threads.FetchMailThread;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 *
 */
public class MailClientMain {

	static Logger	logger	= Logger.getLogger(MailClientMain.class);	// @jve:decl-index=0:

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
		BasicConfigurator.configure();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Mailbox mbx = new Mailbox();
				Config config = new Config();
				try {
					config.readConfig(Config.CONFIG_FILE);
				} catch (IOException e) {
					e.printStackTrace();
				}
				MailClientMain application = new MailClientMain(config, mbx);
				application.fetchMailThread = new Thread(new FetchMailThread(config, mbx));
				application.fetchMailThread.start();
				application.getJFrame().setVisible(true);
			}
		});
		} catch (Exception e) {
			; // ;D
		}
	}

	private JFrame		jFrame				= null; // @jve:decl-index=0:visual-constraint="185,58"

	private JPanel		jContentPane		= null;

	private JMenuBar	jJMenuBar			= null;

	private JMenu		fileMenu			= null;

	private JMenu		editMenu			= null;

	private JMenu		helpMenu			= null;

	private JMenuItem	exitMenuItem		= null;

	private JMenuItem	aboutMenuItem		= null;

	private JMenuItem	cutMenuItem			= null;

	private JMenuItem	copyMenuItem		= null;

	private JMenuItem	pasteMenuItem		= null;

	private JMenuItem	saveMenuItem		= null;

	private JDialog		aboutDialog			= null;

	private JPanel		aboutContentPane	= null;

	private JLabel		aboutVersionLabel	= null;

	private JTextField	jTextFieldStatusBar	= null;

	private JSplitPane	jSplitPane			= null;

	private JScrollPane	jScrollPaneTable	= null;

	private JTable		jTable				= null;

	private JPanel		jPanelMessage		= null;

	private JTextPane	jTextPaneContent	= null;

	private JPanel		jPanelParts			= null;

	private JPanel		jPanelMailHeader	= null;

	private JPanel		jPanelTop			= null;

	private JButton		jButtonQuit			= null;

	private JButton		jButtonGetMails		= null;

	private JTextField	jTextFieldFrom		= null;

	private JTextField	jTextFieldTo		= null;

	private JTextField	jTextFieldSubject	= null;

	private JTextField	jTextFieldDate		= null;

	private Config		config				= null;

	private Mailbox		mailbox				= null;

	private TableModel	tableModel			= null;

	private Message		currentMessage		= null; // @jve:decl-index=0:

	private Thread		fetchMailThread		= null;

	private JButton		jButtonStop			= null;

	private JLabel		subjectLabel		= null;

	private JLabel		fromLabel			= null;

	private JLabel		toLabel				= null;

	private JLabel		ccLabel				= null;

	private JLabel		dateLabel			= null;

	private JTextField	jTextFieldCc		= null;

	private JTextField	subjectText			= null;

	private JTextField	fromText			= null;

	private JTextField	toText				= null;

	private JTextField	ccText				= null;

	private JTextField	dateText			= null;

	private JPanel		headerPanel			= null;

	public MailClientMain(Config config, Mailbox mailbox) {
		this.config = config;
		this.mailbox = mailbox;
		Thread updateTable = new Thread(new Runnable() {
			public void run() {
				while (!Thread.interrupted()) {
					try {
						Thread.sleep(Config.GUI_UPDATE_INTERVAL); // update table every x seconds
						MailClientMain.this.redrawTable();
					} catch (Exception e) {
						;
						e.printStackTrace();
						System.err.println("updateThread exception caught.");
					}
				}
			}
		});
		updateTable.start();
	}

	/**
	 * This method initializes aboutContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getAboutContentPane() {
		if (aboutContentPane == null) {
			aboutContentPane = new JPanel();
			aboutContentPane.setLayout(new BorderLayout());
			aboutContentPane.add(getAboutVersionLabel(), BorderLayout.CENTER);
		}
		return aboutContentPane;
	}

	/**
	 * This method initializes aboutDialog
	 * 
	 * @return javax.swing.JDialog
	 */
	private JDialog getAboutDialog() {
		if (aboutDialog == null) {
			aboutDialog = new JDialog(getJFrame(), true);
			aboutDialog.setTitle("About");
			aboutDialog.setContentPane(getAboutContentPane());
		}
		return aboutDialog;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem();
			aboutMenuItem.setText("About");
			aboutMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JDialog aboutDialog = getAboutDialog();
					aboutDialog.pack();
					Point loc = getJFrame().getLocation();
					loc.translate(20, 20);
					aboutDialog.setLocation(loc);
					aboutDialog.setVisible(true);
				}
			});
		}
		return aboutMenuItem;
	}

	/**
	 * This method initializes aboutVersionLabel
	 * 
	 * @return javax.swing.JLabel
	 */
	private JLabel getAboutVersionLabel() {
		if (aboutVersionLabel == null) {
			aboutVersionLabel = new JLabel();
			aboutVersionLabel.setText("jPOPc Version 0.1a");
			aboutVersionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return aboutVersionLabel;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getCopyMenuItem() {
		if (copyMenuItem == null) {
			copyMenuItem = new JMenuItem();
			copyMenuItem.setText("Copy");
			copyMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK, true));
		}
		return copyMenuItem;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getCutMenuItem() {
		if (cutMenuItem == null) {
			cutMenuItem = new JMenuItem();
			cutMenuItem.setText("Cut");
			cutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK, true));
		}
		return cutMenuItem;
	}

	/**
	 * This method initializes jMenu
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getEditMenu() {
		if (editMenu == null) {
			editMenu = new JMenu();
			editMenu.setText("Edit");
			editMenu.add(getCutMenuItem());
			editMenu.add(getCopyMenuItem());
			editMenu.add(getPasteMenuItem());
		}
		return editMenu;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Exit");
			exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		}
		return exitMenuItem;
	}

	/**
	 * This method initializes jMenu
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.add(getSaveMenuItem());
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}

	private JPanel getHeaderPanel() {
		{
			headerPanel = new JPanel();
			GridBagLayout headerPanelLayout = new GridBagLayout();
			headerPanelLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.1 };
			headerPanelLayout.rowHeights = new int[] { 21, 21, 21, 21, 21 };
			headerPanelLayout.columnWeights = new double[] { 0.0, 0.1 };
			headerPanelLayout.columnWidths = new int[] { 61, 7 };
			headerPanel.setLayout(headerPanelLayout);
			headerPanel.setPreferredSize(new java.awt.Dimension(788, 106));
			{
				subjectLabel = new JLabel();
				headerPanel.add(subjectLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
						GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				subjectLabel.setText("  Subject:");
			}
			{
				fromLabel = new JLabel();
				headerPanel.add(fromLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START,
						GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				fromLabel.setText("  From:");
			}
			{
				toLabel = new JLabel();
				headerPanel.add(toLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START,
						GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				toLabel.setText("  To:");
			}
			{
				ccLabel = new JLabel();
				headerPanel.add(ccLabel, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START,
						GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				ccLabel.setText("  CC:");
			}
			{
				dateLabel = new JLabel();
				headerPanel.add(dateLabel, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START,
						GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				dateLabel.setText("  Date:");
			}
			{
				subjectText = new JTextField();
				subjectText.setBackground(Color.white);
				subjectText.setEditable(false);
				headerPanel.add(subjectText, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
						GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			}
			{
				fromText = new JTextField();
				fromText.setBackground(Color.white);
				fromText.setEditable(false);
				headerPanel.add(fromText, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
						GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			}
			{
				toText = new JTextField();
				toText.setBackground(Color.white);
				toText.setEditable(false);
				headerPanel.add(toText, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
						GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			}
			{
				ccText = new JTextField();
				ccText.setBackground(Color.white);
				ccText.setEditable(false);
				headerPanel.add(ccText, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
						GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			}
			{
				dateText = new JTextField();
				dateText.setBackground(Color.white);
				dateText.setEditable(false);
				headerPanel.add(dateText, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
						GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			}
		}
		return headerPanel;
	}

	/**
	 * This method initializes jMenu
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getHelpMenu() {
		if (helpMenu == null) {
			helpMenu = new JMenu();
			helpMenu.setText("Help");
			helpMenu.add(getAboutMenuItem());
		}
		return helpMenu;
	}

	/**
	 * This method initializes jButton1
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonGetMails() {
		if (jButtonGetMails == null) {
			jButtonGetMails = new JButton();
			jButtonGetMails.setText("Get Mails");
			jButtonGetMails.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getClickCount() >= 1) {
						Thread th = new Thread(new Runnable() {
							public void run() {
								try {
									MailClientMain.this.fetchMailThread.join();
									MailClientMain.this.fetchMailThread = new Thread(new FetchMailThread(
											MailClientMain.this.config, MailClientMain.this.mailbox));
									MailClientMain.this.fetchMailThread.start();
								} catch (InterruptedException e) {
									e.printStackTrace();
								}
							}
						});
						th.start();

					}
				}
			});
		}
		return jButtonGetMails;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonQuit() {
		if (jButtonQuit == null) {
			jButtonQuit = new JButton();
			jButtonQuit.setText("Quit");
			jButtonQuit.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getClickCount() >= 1) {
						System.exit(0);
					}
				}
			});
		}
		return jButtonQuit;
	}

	/**
	 * This method initializes jButtonStop
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonStop() {
		if (jButtonStop == null) {
			jButtonStop = new JButton();
			jButtonStop.setText("Stop");
			jButtonStop.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getClickCount() >= 1) {
						MailClientMain.this.fetchMailThread.interrupt();
					}
				}
			});
		}
		return jButtonStop;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJTextFieldStatusBar(), BorderLayout.SOUTH);
			jContentPane.add(getJPanelTop(), BorderLayout.NORTH);
			jContentPane.add(getJSplitPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jFrame
	 * 
	 * @return javax.swing.JFrame
	 */
	private JFrame getJFrame() {
		if (jFrame == null) {
			jFrame = new JFrame();
			jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			jFrame.setLocation(new Point(50, 50));
			jFrame.setPreferredSize(new Dimension(800, 600));
			jFrame.setJMenuBar(getJJMenuBar());
			jFrame.setSize(800, 600);
			jFrame.setContentPane(getJContentPane());
			jFrame.setTitle("jPOPc");
		}
		return jFrame;
	}

	/**
	 * This method initializes jJMenuBar
	 * 
	 * @return javax.swing.JMenuBar
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getFileMenu());
			jJMenuBar.add(getEditMenu());
			jJMenuBar.add(getHelpMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jPanel2
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelMailHeader() {
		if (jPanelMailHeader == null) {
			jPanelMailHeader = new JPanel();
			jPanelMailHeader = getHeaderPanel();
			/*
			 * jPanelMailHeader.setLayout(new BoxLayout(getJPanelMailHeader(), BoxLayout.Y_AXIS));
			 * jPanelMailHeader.add(getJPanelFrom(), null); jPanelMailHeader.add(getJPanelTo(), null);
			 * jPanelMailHeader.add(getJPanelCc(), null); jPanelMailHeader.add(getJPanelSubject(), null);
			 * jPanelMailHeader.add(getJPanelDate(), null);
			 */
		}
		return jPanelMailHeader;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelMessage() {
		if (jPanelMessage == null) {
			jPanelMessage = new JPanel();
			jPanelMessage.setLayout(new BorderLayout());
			jPanelMessage.add(getJTextPaneContent(), BorderLayout.CENTER);
			jPanelMessage.add(getJPanelParts(), BorderLayout.SOUTH);
			jPanelMessage.add(getJPanelMailHeader(), BorderLayout.NORTH);
		}
		return jPanelMessage;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelParts() {
		if (jPanelParts == null) {
			jPanelParts = new JPanel();
			jPanelParts.setLayout(new BoxLayout(getJPanelParts(), BoxLayout.X_AXIS));
		}
		return jPanelParts;
	}

	/**
	 * This method initializes jPanelTop
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelTop() {
		if (jPanelTop == null) {
			jPanelTop = new JPanel();
			jPanelTop.setLayout(new BoxLayout(getJPanelTop(), BoxLayout.X_AXIS));
			jPanelTop.add(getJButtonGetMails(), null);
			jPanelTop.add(getJButtonStop(), null);
			jPanelTop.add(getJButtonQuit(), null);
		}
		return jPanelTop;
	}

	/**
	 * This method initializes jScrollPaneTable
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPaneTable() {
		if (jScrollPaneTable == null) {
			jScrollPaneTable = new JScrollPane();
			jScrollPaneTable.setViewportView(getJTable());
		}
		return jScrollPaneTable;
	}

	/**
	 * This method initializes jSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			jSplitPane.setDividerLocation(125); // set this to a reasonable amount, to prevent the table from being tiny
			jSplitPane.setLeftComponent(getJScrollPaneTable());
			jSplitPane.setRightComponent(getJPanelMessage());
		}
		return jSplitPane;
	}

	/**
	 * This method initializes jTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable() {
		if (jTable == null) {
			this.tableModel = new DefaultTableModel(this.mailbox.getTableData(), this.mailbox.getTableHeaders());
			jTable = new JTable(tableModel);
			jTable.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getClickCount() > 0) {
						MailClientMain.this.showMail(((JTable) e.getSource()).rowAtPoint(e.getPoint()));
					}
				}
			});
		}
		return jTable;
	}

	/**
	 * This method initializes jTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextFieldStatusBar() {
		if (jTextFieldStatusBar == null) {
			jTextFieldStatusBar = new JTextField();
			jTextFieldStatusBar.setText("Please wait, starting. Table and Statusbar are updated every "
					+ (Config.GUI_UPDATE_INTERVAL / 1000) + " seconds.");
		}
		return jTextFieldStatusBar;
	}

	/**
	 * This method initializes jTextPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getJTextPaneContent() {
		if (jTextPaneContent == null) {
			jTextPaneContent = new JTextPane();
			jTextPaneContent.setAutoscrolls(true);
		}
		return jTextPaneContent;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getPasteMenuItem() {
		if (pasteMenuItem == null) {
			pasteMenuItem = new JMenuItem();
			pasteMenuItem.setText("Paste");
			pasteMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, Event.CTRL_MASK, true));
		}
		return pasteMenuItem;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getSaveMenuItem() {
		if (saveMenuItem == null) {
			saveMenuItem = new JMenuItem();
			saveMenuItem.setText("Save");
			saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK, true));
		}
		return saveMenuItem;
	}

	/**
	 * Redraw the JTable to reflect the (possible) changes in the data model.
	 * 
	 */
	protected void redrawTable() {
		TableModel newTable = new DefaultTableModel(this.mailbox.getTableData(), this.mailbox.getTableHeaders());
		if (this.tableModel.equals(newTable)) {
			logger.trace("Not updating the table, nothing has changed.");
		} else {
			this.tableModel = new DefaultTableModel(this.mailbox.getTableData(), this.mailbox.getTableHeaders());
			this.jTable.setModel(this.tableModel);
			this.jTable.validate();
			if (this.fetchMailThread.isAlive()) {
				this.jTextFieldStatusBar.setText("Busy ...");
			} else {
				this.jTextFieldStatusBar.setText("Ready.");
			}
		}
	}

	/**
	 * Save part from a multipart mail. This method is invoced for every part whose content-type doesn't start with
	 * "text".
	 * 
	 * @param infoId
	 */
	protected void savePart(int infoId) {
		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(new File(this.currentMessage.getPart(infoId).getHeader()
				.getContentDispositionFilename()));
		File outFile = null;
		int returnVal = chooser.showSaveDialog(this.jContentPane);
		if (returnVal == JFileChooser.ERROR_OPTION) {
			logger.error("savePart - FileChooser error");
		}
		try {
			outFile = new File(chooser.getSelectedFile().getAbsolutePath());
		} catch (NullPointerException e) {
			return; // since no file was choosen, the user didn't wan't us to save the file
		}
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(outFile);
			byte[] outBuffer = this.currentMessage.getPart(infoId).getRawContent();
			for (int i = 0; i < outBuffer.length; i++) {
				fos.write(outBuffer[i]);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				fos.close();
			} catch (IOException e) {
				;
			}
		}

	}

	/**
	 * Show a specific mail
	 * 
	 * @param i
	 */
	protected void showMail(int i) {
		this.currentMessage = this.mailbox.getMessage(i);
		if (this.currentMessage != null) {
			this.fromText.setText(this.currentMessage.getHeader().getFrom());
			this.toText.setText(this.currentMessage.getHeader().getTo());
			this.ccText.setText(this.currentMessage.getHeader().getCc());
			this.dateText.setText(this.currentMessage.getHeader().getDate().toString());
			this.subjectText.setText(this.currentMessage.getHeader().getSubject());
			this.jPanelMessage.remove(this.jTextPaneContent);
			if (this.currentMessage.getHeader().getContentType().toLowerCase().equals("text/html".toLowerCase())) {
				// if the content pane isn't already displaying html we have to create a new one.
				if (!this.jTextPaneContent.getContentType().toLowerCase().equals("text/html".toLowerCase()))
					this.jTextPaneContent = new JTextPane();
				this.jTextPaneContent.setDocument(new HTMLEditorKit().createDefaultDocument());
				this.jTextPaneContent.setContentType("text/html");
			} else {
				// if the content pane isn't already displaying text we have to create a new one.
				if (this.jTextPaneContent.getContentType().toLowerCase().equals("text/html".toLowerCase()))
					this.jTextPaneContent = new JTextPane();
				this.jTextPaneContent.setContentType("text/plain");
			}
			if (true || this.currentMessage.getNumParts() > 1) {
				this.jPanelParts.removeAll();
				this.jPanelParts.repaint();
				for (int j = 0; j < this.currentMessage.getNumParts(); j++) {
					JButtonWithInfo btn = new JButtonWithInfo();
					btn.setText(this.currentMessage.getPart(j).getContentType());
					btn.setInfoId(j); // store the number of the current part
					// if it isn't text we can't display it, so we enable the user to save it
					if (this.currentMessage.getPart(j).getContentType().toLowerCase().startsWith("text".toLowerCase())) {
						btn.addMouseListener(new MouseAdapter() {
							public void mouseClicked(MouseEvent e) {
								if (e.getClickCount() >= 1) {
									MailClientMain.this.switchContentTypeTo(((JButtonWithInfo) e.getSource())
											.getInfoId());
								}
							}
						});
					} else {
						btn.addMouseListener(new MouseAdapter() {
							public void mouseClicked(MouseEvent e) {
								if (e.getClickCount() >= 1) {
									MailClientMain.this.savePart(((JButtonWithInfo) e.getSource()).getInfoId());
								}
							}
						});
					}
					this.jPanelParts.add(btn);
				}

				// add two more buttons: one for the parsed header and one for the raw header
				JButtonWithInfo btn = new JButtonWithInfo();
				btn.setText("Header");
				btn.setInfoId(-23); // use the "magic" number -23 to indicate that we want to get the parsed header
				btn.addMouseListener(new MouseAdapter() {
					public void mouseClicked(MouseEvent e) {
						if (e.getClickCount() >= 1) {
							MailClientMain.this.switchContentTypeTo(((JButtonWithInfo) e.getSource()).getInfoId());
						}
					}
				});
				this.jPanelParts.add(btn);
				btn = null;
				btn = new JButtonWithInfo();
				btn.setText("RAW-Header");
				btn.setInfoId(-42); // use the "magic" number -42 to indicate that we want to get the raw header
				btn.addMouseListener(new MouseAdapter() {
					public void mouseClicked(MouseEvent e) {
						if (e.getClickCount() >= 1) {
							MailClientMain.this.switchContentTypeTo(((JButtonWithInfo) e.getSource()).getInfoId());
						}
					}
				});
				this.jPanelParts.add(btn);
				this.jPanelParts.validate();
				this.jTextPaneContent.setText(this.currentMessage.getPart(0).getContent());
			} else {
				this.jPanelParts.removeAll();
				this.jPanelParts.validate();
				this.jTextPaneContent.setText(this.currentMessage.getPart(0).getContent());
			}
			this.jPanelMessage.add(this.jTextPaneContent);
			this.jTextPaneContent.validate();
			this.jPanelMessage.validate();
		} else {
			// blank everything
			this.jPanelParts.removeAll();
			this.jPanelParts.validate();
			this.jTextFieldFrom.setText("");
			this.jTextFieldTo.setText("");
			this.jTextFieldCc.setText("");
			this.jTextFieldDate.setText("");
			this.jTextFieldSubject.setText("");
			this.jTextPaneContent.setText("");
		}
	}

	/**
	 * Show a diffent part of the currently selected mail. This will only work if it's a multipart mail.
	 * 
	 * @param id
	 */
	protected void switchContentTypeTo(int id) {
		logger.trace("Requested context type switch to: " + id);
		this.jPanelMessage.remove(this.jTextPaneContent);
		this.jTextPaneContent = new JTextPane();
		if (id > -1) { // only positive part numbers are possible
			// because we start at part no. 0 when counting,
			// everything below 0 has a special meaning
			if (this.currentMessage.getPart(id).getContentType().toLowerCase().equals("text/html".toLowerCase())) {
				this.jTextPaneContent.setDocument(new HTMLEditorKit().createDefaultDocument());
				this.jTextPaneContent.setContentType("text/html");
			} else {
				this.jTextPaneContent.setContentType("text/plain");
			}
			this.jTextPaneContent.setText(this.currentMessage.getPart(id).getContent());

		} else { // this id is lower than 0
			// this.jTextPaneContent.setContentType("text/plain");
			if (id == -23) { // show the parsed header
				this.jTextPaneContent.setText(this.currentMessage.getHeader().toString());
			} else if (id == -42) { // show the raw header
				this.jTextPaneContent.setText(this.currentMessage.getHeader().getRawHeader());
			} else {
				this.jTextPaneContent.setText("");
				logger.warn("Illegal Part Number requested: " + id);
			}
		}
		this.jPanelMessage.add(this.jTextPaneContent);
		this.jPanelMessage.validate();
	}

}
