package tests.core;

import junit.framework.TestCase;
import core.util.Decode;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 *
 */
public class DecodeTest extends TestCase {

	private String	messageBase64		= "SW4gZGVyIFF1YW50ZW5weWhzaWsgYmVzYWd0IGRpZSBVbnNjaMOkcmZlcmVsYXRpb24sIGRhc3Mg\n"
												+ "bmljaHQgZ2xlaWNoemVpdGlnIE9ydCB1bmQgSW1wdWxzIGVpbmVzIFRlaWxjaGVucyBnZW5hdSBi\n"
												+ "ZXN0aW1tdCB3ZXJkZW4ga8O2bm5lbi4gQmVpIGVpbmVtIE9ydCB4IHVuZCBlaW5lbSBJbXB1bHMg\n"
												+ "cCBleGlzdGllcnQgaW1tZXIgZWluZSBzb2dlbmFubnRlIE9ydHN1bnNjaMOkcmZlIM6UeCB1bmQg\n"
												+ "ZWluZSBJbXB1bHN1bnNjaMOkcmZlIM6UcCwgendpc2NoZW4gZGVuZW4gZm9sZ2VuZGUgUmVsYXRp\n"
												+ "b24gYmVzdGVodDoNCg0KzpR4zpRwIOKJpSBoLzTPgCA9IMSnLzINCg0KRGFiZWkgaXN0IHBsYW5j\n"
												+ "a3NjaGUgV2lya3VuZ3NxdWFudHVtIGggPSA2LDYyNjHCtzEwXi0zNCBKcy4NCg0KTWl0IGZyZXVu\n"
												+ "ZGxpY2hlbiBHcsO8w59lbi4NCg==";

	private String	messageBase64Clear	= "In der Quantenpyhsik besagt die Unschärferelation, dass nicht gleichzeitig Ort und Impuls eines Teilchens genau bestimmt werden können. Bei einem Ort x und einem Impuls p existiert immer eine sogenannte Ortsunschärfe Δx und eine Impulsunschärfe Δp, zwischen denen folgende Relation besteht:\n"
												+ "\n"
												+ "ΔxΔp ≥ h/4π = ħ/2\n"
												+ "\n"
												+ "Dabei ist plancksche Wirkungsquantum h = 6,6261·10-34 Js.\n"
												+ "\n"
												+ "Mit freundlichen Grüßen.\n";

	private String	messageQP			= "Dar=FCber hinaus verwenden die meisten europ=E4ischen Sprachen noch weiter=\n"
												+ "e Zeichen,\n"
												+ "die sogenannten \"diakritischen Zeichen\". Bekannt sind die Akzente wie =E9 =\n"
												+ "und =E8, die in\n"
												+ "vielen Sprachen zur Kennzeichnung der Betonung verwendet werden, die C=E9d=\n"
												+ "ille (=E7),\n"
												+ "aus den Romanischen Sprachen oder das spanische enje (=F1).";

	private String	messageQPClear		= "Darüber hinaus verwenden die meisten europäischen Sprachen noch weitere Zeichen,\n"
												+ "die sogenannten \"diakritischen Zeichen\". Bekannt sind die Akzente wie é und è, die in\n"
												+ "vielen Sprachen zur Kennzeichnung der Betonung verwendet werden, die Cédille (ç),\n"
												+ "aus den Romanischen Sprachen oder das spanische enje (ñ).";

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testDecodeBase64() {
		assertEquals(messageBase64Clear, Decode.decodeBase64(messageBase64, "utf-8"));
	}

	public void testDecodeQuotedPrintable() {
		assertEquals(messageQPClear, Decode.decodeQuotedPrintable(messageQP, "utf-8"));
	}

	public void testEmptyString() {
		assertEquals("", Decode.decodeBase64("", "utf-8"));
	}

	public void testFString() {
		assertEquals("f", Decode.decodeBase64("Zg==", "utf-8"));
	}

	public void testFoString() {
		assertEquals("fo", Decode.decodeBase64("Zm8=", "utf-8"));
	}

	public void testFooString() {
		assertEquals("foo", Decode.decodeBase64("Zm9v", "utf-8"));
	}

	public void testFoobString() {
		assertEquals("foob", Decode.decodeBase64("Zm9vYg==", "utf-8"));
	}

	public void testFoobaString() {
		assertEquals("fooba", Decode.decodeBase64("Zm9vYmE=", "utf-8"));
	}

	public void testFoobarString() {
		assertEquals("foobar", Decode.decodeBase64("Zm9vYmFy", "utf-8"));
	}

}
