/*
 * Created on 02.11.2006
 * 
 */
package tests.core;

import junit.framework.TestCase;
import core.structures.Header;
import core.util.StringTools;

/**
 * 
 * <pre>
 *  Copyright (c) 2006 Dominik Schulz
 *  Copyright (c) 2006 Florian Lindner
 *  Copyright (c) 2006 Philip Hartmann
 *  
 *  This file is part of jPOPc.
 *
 *  jPOPc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jPOPc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with jPOPc; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *  
 * @author Dominik
 *
 */
public class HeaderTest extends TestCase {

	private Header	hd				= null;

	private String	rawHeader		= "Received: from mail by mail.gauner.org with spam-scanned (Exim 4.50)\n"
											+ "   id 1GfaoJ-00066t-Ai\n"
											+ "   for lkml@ds.gauner.org; Thu, 02 Nov 2006 12:32:46 +0100\n"
											+ "Received: from vger.kernel.org ([209.132.176.167])\n"
											+ "   by mail.gauner.org with esmtp (Exim 4.50)\n"
											+ "   id 1GfaoI-00066o-U4\n"
											+ "   for lkml@ds.gauner.org; Thu, 02 Nov 2006 12:32:39 +0100\n"
											+ "Received: (majordomo@vger.kernel.org) by vger.kernel.org via listexpand\n"
											+ "   id S1752054AbWKBLcW (ORCPT <rfc822;lkml@ds.gauner.org>);\n"
											+ "   Thu, 2 Nov 2006 06:32:22 -0500\n"
											+ "Received: (majordomo@vger.kernel.org) by vger.kernel.org id S1752833AbWKBLcW\n"
											+ "   (ORCPT <rfc822;linux-kernel-outgoing>);\n"
											+ "   Thu, 2 Nov 2006 06:32:22 -0500\n"
											+ "Received: from rumms.uni-mannheim.de ([134.155.50.52]:28802 \"EHLO\n"
											+ "   rumms.uni-mannheim.de\") by vger.kernel.org with ESMTP\n"
											+ "   id S1752054AbWKBLcV (ORCPT <rfc822;linux-kernel@vger.kernel.org>);\n"
											+ "   Thu, 2 Nov 2006 06:32:21 -0500\n"
											+ "Received: from [134.155.65.188] (eml-pc3.informatik.uni-mannheim.de [134.155.65.188])\n"
											+ "   (authenticated bits=0)\n"
											+ "   by rumms.uni-mannheim.de (8.13.8/8.13.8) with ESMTP id kA2BVjrO012099;\n"
											+ "   Thu, 2 Nov 2006 12:31:57 +0100 (MET)\n"
											+ "Message-ID: <4549D722.5090007@ti.uni-mannheim.de>\n"
											+ "Date:  Thu, 02 Nov 2006 12:31:46 +0100\n"
											+ "From:  Guillermo Marcus <marcus@ti.uni-mannheim.de>\n"
											+ "User-Agent: Thunderbird 1.5.0.7 (X11/20060911)\n"
											+ "MIME-Version: 1.0\n"
											+ "To:    rmk+lkml@arm.linux.org.uk\n"
											+ "CC:    linux-kernel@vger.kernel.org\n"
											+ "Subject: Re: mmaping a kernel buffer to user space\n"
											+ "References: <4547150F.8070408@ti.uni-mannheim.de> <loom.20061101T120846-320@post.gmane.org> <454899E9.1090900@ti.uni-mannheim.de> <20061102083109.GB1377@flint.arm.linux.org.uk>\n"
											+ "In-Reply-To: <20061102083109.GB1377@flint.arm.linux.org.uk>\n"
											+ "X-Enigmail-Version: 0.94.1.0\n"
											+ "Content-Type: text/plain; charset=ISO-8859-1\n"
											+ "Content-Transfer-Encoding: 7bit\n"
											+ "Sender:    linux-kernel-owner@vger.kernel.org\n"
											+ "Precedence: bulk\n"
											+ "X-Mailing-List:    linux-kernel@vger.kernel.org\n"
											+ "X-Spam-Checker-Version: SpamAssassin 3.0.3 (2005-04-27) on mail.gauner.org\n"
											+ "X-Spam-Level: \n"
											+ "X-Spam-Status: No, score=-2.6 required=5.0 tests=AWL,BAYES_00 autolearn=ham \n"
											+ "   version=3.0.3";

	private String	unfoldedHeader	= "Received: from mail by mail.gauner.org with spam-scanned (Exim 4.50) id 1GfaoJ-00066t-Ai for lkml@ds.gauner.org; Thu, 02 Nov 2006 12:32:46 +0100\n"
											+ "Received: from vger.kernel.org ([209.132.176.167]) by mail.gauner.org with esmtp (Exim 4.50) id 1GfaoI-00066o-U4 for lkml@ds.gauner.org; Thu, 02 Nov 2006 12:32:39 +0100\n"
											+ "Received: (majordomo@vger.kernel.org) by vger.kernel.org via listexpand id S1752054AbWKBLcW (ORCPT <rfc822;lkml@ds.gauner.org>); Thu, 2 Nov 2006 06:32:22 -0500\n"
											+ "Received: (majordomo@vger.kernel.org) by vger.kernel.org id S1752833AbWKBLcW (ORCPT <rfc822;linux-kernel-outgoing>);Thu, 2 Nov 2006 06:32:22 -0500\n"
											+ "Received: from rumms.uni-mannheim.de ([134.155.50.52]:28802 \"EHLO rumms.uni-mannheim.de\") by vger.kernel.org with ESMTP id S1752054AbWKBLcV (ORCPT <rfc822;linux-kernel@vger.kernel.org>);Thu, 2 Nov 2006 06:32:21 -0500\n"
											+ "Received: from [134.155.65.188] (eml-pc3.informatik.uni-mannheim.de [134.155.65.188]) (authenticated bits=0) by rumms.uni-mannheim.de (8.13.8/8.13.8) with ESMTP id kA2BVjrO012099;Thu, 2 Nov 2006 12:31:57 +0100 (MET)\n"
											+ "Message-ID: <4549D722.5090007@ti.uni-mannheim.de>\n"
											+ "Date:  Thu, 02 Nov 2006 12:31:46 +0100\n"
											+ "From:  Guillermo Marcus <marcus@ti.uni-mannheim.de>\n"
											+ "User-Agent: Thunderbird 1.5.0.7 (X11/20060911)\n"
											+ "MIME-Version: 1.0\n"
											+ "To:    rmk+lkml@arm.linux.org.uk\n"
											+ "CC:    linux-kernel@vger.kernel.org\n"
											+ "Subject: Re: mmaping a kernel buffer to user space\n"
											+ "References: <4547150F.8070408@ti.uni-mannheim.de> <loom.20061101T120846-320@post.gmane.org> <454899E9.1090900@ti.uni-mannheim.de> <20061102083109.GB1377@flint.arm.linux.org.uk>\n"
											+ "In-Reply-To: <20061102083109.GB1377@flint.arm.linux.org.uk>\n"
											+ "X-Enigmail-Version: 0.94.1.0\n"
											+ "Content-Type: text/plain; charset=ISO-8859-1\n"
											+ "Content-Transfer-Encoding: 7bit\n"
											+ "Sender:    linux-kernel-owner@vger.kernel.org\n"
											+ "Precedence: bulk\n"
											+ "X-Mailing-List:    linux-kernel@vger.kernel.org\n"
											+ "X-Spam-Checker-Version: SpamAssassin 3.0.3 (2005-04-27) on mail.gauner.org\n"
											+ "X-Spam-Level: \n"
											+ "X-Spam-Status: No, score=-2.6 required=5.0 tests=AWL,BAYES_00 autolearn=ham version=3.0.3";

	protected void setUp() throws Exception {
		this.hd = new Header();
		this.hd.setRawHeader(rawHeader);
		super.setUp();
	}

	protected void tearDown() throws Exception {
		this.hd = null;
		super.tearDown();
	}

	public void testGetContentTransferEncoding() {
		assertEquals("7bit", hd.getContentTransferEncoding());
	}

	public void testSetContentTransferEncoding() {
		String cte = "8bit";
		hd.setContentTransferEncoding(cte);
		assertEquals(cte, hd.getContentTransferEncoding());
	}

	public void testGetCc() {
		assertEquals("linux-kernel@vger.kernel.org", hd.getCc());
		System.out.println(hd.getCc());
	}

	public void testSetCc() {
		String cc = "test@test.com";
		hd.setCc(cc);
		assertEquals(cc, hd.getCc());
	}

	public void testSetDate() {
		fail("Not yet implemented");
	}

	public void testGetFrom() {
		assertEquals("Guillermo Marcus <marcus@ti.uni-mannheim.de>", hd.getFrom());
		System.out.println(hd.getFrom());
	}

	public void testSetFrom() {
		String from = "test@test.com";
		hd.setFrom(from);
		assertEquals(from, hd.getFrom());
	}

	public void testGetRawHeader() {
		assertEquals(rawHeader, hd.getRawHeader());
	}

	public void testParseDate() {
		fail("Not yet implemented");
	}

	public void testGetSender() {
		assertEquals("linux-kernel-owner@vger.kernel.org", hd.getSender());
	}

	public void testSetSender() {
		String sender = "test@test.com";
		hd.setSender(sender);
		assertEquals(sender, hd.getSender());
	}

	public void testGetSubject() {
		assertEquals("Re: mmaping a kernel buffer to user space", hd.getSubject());
	}

	public void testSetSubject() {
		String subj = "This is a really nice Test-Subject. �߰!\"�$%&/()=?`^+*~'-_.:,';<>|@��";
		hd.setSubject(subj);
		assertEquals(subj, hd.getSubject());
	}

	public void testGetTo() {
		assertEquals("rmk+lkml@arm.linux.org.uk", hd.getTo());
	}

	public void testSetTo() {
		String to = "test@test.com";
		hd.setTo(to);
		assertEquals(to, hd.getTo());
	}

	public void testGetContentType() {
		assertEquals("text/plain", hd.getContentType());
	}

	public void testSetContentType() {
		String ct = "text/plain";
		hd.setContentType(ct);
		assertEquals(ct, hd.getContentType());
	}

	public void testUnfoldHeaderLinesArrayListOfString() {
		assertEquals(unfoldedHeader, Header.unfoldHeaderLines(StringTools.stringToArrayList(rawHeader)));
	}

	public void testUnfoldHeaderLinesStringArray() {
		assertEquals(unfoldedHeader, Header.unfoldHeaderLines(StringTools.stringToArray(rawHeader)));
	}

}
